using System;
using LiteNetLib.Utils;
using Microsoft.Xna.Framework.Input;

using Syntriax.Engine.Core;
using Syntriax.Engine.Input;
using Syntriax.Engine.Network;
using Syntriax.Engine.Network.Abstract;

namespace Pong.Behaviours;

public class PaddleBehaviour(Keys Up, Keys Down, float High, float Low, float Speed) : NetworkBehaviour
{
    private Keys Up { get; } = Up;
    private Keys Down { get; } = Down;
    public float High { get; } = High;
    public float Low { get; } = Low;
    public float Speed { get; set; } = Speed;

    private bool isUpPressed = false;
    private bool isDownPressed = false;

    private IButtonInputs<Keys> inputs = null!;

    protected override void OnUpdate()
    {
        if (isUpPressed && isDownPressed)
            return;

        if (isUpPressed)
            Move(Vector2D.Up);
        else if (isDownPressed)
            Move(-Vector2D.Up);
    }

    private void Move(Vector2D vectorToMove)
    {
        GameObject.Transform.Position = GameObject.Transform.Position + vectorToMove * (float)Time.Elapsed.TotalSeconds * Speed;
        GameObject.Transform.Position = new Vector2D(GameObject.Transform.Position.X, MathF.Max(MathF.Min(GameObject.Transform.Position.Y, High), Low));
    }

    protected override void OnFirstActiveFrame()
    {
        if (!BehaviourController.TryGetBehaviour<IButtonInputs<Keys>>(out var behaviourResult))
            inputs = behaviourResult ?? BehaviourController.AddBehaviour<KeyboardInputsBehaviour>();

        inputs.RegisterOnPress(Up, OnUpPressed);
        inputs.RegisterOnRelease(Up, OnUpReleased);

        inputs.RegisterOnPress(Down, OnDownPressed);
        inputs.RegisterOnRelease(Down, OnDownReleased);
    }

    protected override void OnFinalize()
    {
        inputs.UnregisterOnPress(Up, OnUpPressed);
        inputs.UnregisterOnRelease(Up, OnUpReleased);

        inputs.UnregisterOnPress(Down, OnDownPressed);
        inputs.UnregisterOnRelease(Down, OnDownReleased);
    }

    private void OnUpPressed(IButtonInputs<Keys> inputs, Keys keys) { isUpPressed = true; SendData(new PaddleInputs() { IsUpPressed = isUpPressed, IsDownPressed = isDownPressed }); }
    private void OnUpReleased(IButtonInputs<Keys> inputs, Keys keys) { isUpPressed = false; SendData(new PaddleInputs() { IsUpPressed = isUpPressed, IsDownPressed = isDownPressed }); }
    private void OnDownPressed(IButtonInputs<Keys> inputs, Keys keys) { isDownPressed = true; SendData(new PaddleInputs() { IsUpPressed = isUpPressed, IsDownPressed = isDownPressed }); }
    private void OnDownReleased(IButtonInputs<Keys> inputs, Keys keys) { isDownPressed = false; SendData(new PaddleInputs() { IsUpPressed = isUpPressed, IsDownPressed = isDownPressed }); }

    public override void ReceiveData<T>(T data)
    {
        if (data is not PaddleInputs paddleInputs)
            return;

        isUpPressed = paddleInputs.IsUpPressed;
        isDownPressed = paddleInputs.IsDownPressed;
    }

    [System.Serializable]
    public struct PaddleInputs : INetworkPacket
    {
        public bool IsUpPressed { get; set; }
        public bool IsDownPressed { get; set; }

        public void Deserialize(NetDataReader reader)
        {
            IsUpPressed = reader.GetBool();
            IsDownPressed = reader.GetBool();
        }

        public void Serialize(NetDataWriter writer)
        {
            writer.Put(IsUpPressed);
            writer.Put(IsDownPressed);
        }
    }
}
